@@ -7,20 +7,23 @@ import sys
 from copy import deepcopy
 from inspect import Parameter, signature, isclass, isfunction
 from typing import Any, Dict, Union
+from socket import gethostname
 
-_PTC = "cfg://"
-_NPR = len(_PTC)
-_SEP = "/"
-_MOD = "__module__"
-_CLS = "__class__"
-_BSE = "__base__"
 _AST = "__as_type__"
+_BSE = "__base__"
+_CLS = "__class__"
+_HST = "__hostname__"
+_MOD = "__module__"
+_PRT = ".."  # parent
+_PTC = "cfg://"  # protocal
+_PTL = len(_PTC)  # protocal length
+_SEP = "/"  # seperator
 
 undefined = f"{_PTC}__undefined__"
 
 def _remove_protocol_prefix(uri):
     if uri.startswith(_PTC):
-        uri = uri[_NPR:]
+        uri = uri[_PTL:]
     return uri
 
 class CfgOptParseError(Exception): ...
@@ -83,12 +86,14 @@ class ConfigContainer:
             return [uri]
         # remove optional 'cfg://' prefix
         uri = _remove_protocol_prefix(uri)
-        # eliminate ".." in dict hiearchy keys
-        def eliminate_parent(_keys):
+        # translate special keys in dict hiearchy keys
+        def translate_uri(_keys):
             out = []
             for k in _keys:
-                if k == "..":
+                if k == _PRT:
                     out.pop()
+                elif k == _HST:
+                    out.append(gethostname())
                 else:
                     out.append(k)
             return out
@@ -100,12 +105,18 @@ class ConfigContainer:
             keys = [uri[:addr_sep]]
             if addr_sep == len(uri) or addr_sep == len(uri) - 1:
                 return keys
-            keys.extend(eliminate_parent(uri[addr_sep+1:].split(_SEP)))
+            keys.extend(translate_uri(uri[addr_sep+1:].split(_SEP)))
         else:
-            keys = eliminate_parent(uri.split(_SEP))
+            keys = translate_uri(uri.split(_SEP))
         
         return keys
     
+    def __getstate__(self):
+        return self.data
+
+    def __setstate__(self, data):
+        self.data = data
+    
     def __getattr__(self, attrname):
         return getattr(self.data, attrname)
     
@@ -187,7 +198,9 @@ def parse_configs(cfg_root:Union[str, Dict], args=None) -> ConfigContainer:
     updates it with command line options, follows and substitute
     the `cfg://` block references."""
 
-    if isinstance(cfg_root, str) and osp.isdir(cfg_root):
+    if isinstance(cfg_root, str):
+        if not osp.isdir(cfg_root):
+            raise TypeError(f"`cfg_root` (\"{cfg_root}\") is not a directory.")
         # load raw data from json files
         root = {}
         cfg_file_glob_pattern = osp.join(cfg_root, "**", "*.json")
@@ -300,7 +313,10 @@ def parse_configs(cfg_root:Union[str, Dict], args=None) -> ConfigContainer:
                 if k.startswith("__"): continue
                 data[k] = parse_python_objects(data[k])
             if _MOD in data and _CLS in data:
-                module = importlib.import_module(data[_MOD])
+                try:
+                    module = importlib.import_module(data[_MOD])
+                except ModuleNotFoundError as e:
+                    raise ModuleNotFoundError(e, data[_MOD])
                 klass = getattr(module, data[_CLS])
 
                 # fill omitted params with defaults
@@ -316,7 +332,10 @@ def parse_configs(cfg_root:Union[str, Dict], args=None) -> ConfigContainer:
             data = [parse_python_objects(_) for _ in data]
         return data
 
-    parse_python_objects(root)
+    try:
+        parse_python_objects(root)
+    except ModuleNotFoundError as e:
+        raise CfgOptParseError(f"Trying to import '{e.args[1]}', but n{str(e.args[0])[1:]}. Check your PYTHONPATH.") from None
 
     # command line update again (this time including expanded and inherited fields.)
     unparsed_args = command_line_update(unparsed_args)
